package com.sun.org.apache.xml.internal.serialize;

import java.lang.reflect.Method;
import java.io.UnsupportedEncodingException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.io.OutputStream;
import com.sun.org.apache.xerces.internal.util.EncodingMap;

public class EncodingInfo
{
    private Object[] fArgsForMethod;
    String ianaName;
    String javaName;
    int lastPrintable;
    Object fCharsetEncoder;
    Object fCharToByteConverter;
    boolean fHaveTriedCToB;
    boolean fHaveTriedCharsetEncoder;
    
    public EncodingInfo(String ianaName, String javaName, int lastPrintable) {
        this.fArgsForMethod = null;
        this.fCharsetEncoder = null;
        this.fCharToByteConverter = null;
        this.fHaveTriedCToB = false;
        this.fHaveTriedCharsetEncoder = false;
        this.ianaName = ianaName;
        this.javaName = EncodingMap.getIANA2JavaMapping(ianaName);
        this.lastPrintable = lastPrintable;
    }
    
    public String getIANAName() {
        return this.ianaName;
    }
    
    public Writer getWriter(OutputStream output) throws UnsupportedEncodingException {
        if (this.javaName != null) {
            return new OutputStreamWriter(output, this.javaName);
        }
        this.javaName = EncodingMap.getIANA2JavaMapping(this.ianaName);
        if (this.javaName == null) {
            return new OutputStreamWriter(output, "UTF8");
        }
        return new OutputStreamWriter(output, this.javaName);
    }
    
    public boolean isPrintable(char ch) {
        return ch <= this.lastPrintable || this.isPrintable0(ch);
    }
    
    private boolean isPrintable0(char ch) {
        if (this.fCharsetEncoder == null && EncodingInfo.CharsetMethods.fgNIOCharsetAvailable && !this.fHaveTriedCharsetEncoder) {
            if (this.fArgsForMethod == null) {
                this.fArgsForMethod = new Object[1];
            }
            try {
                this.fArgsForMethod[0] = this.javaName;
                Object charset = EncodingInfo.CharsetMethods.fgCharsetForNameMethod.invoke(null, this.fArgsForMethod);
                if ((boolean)EncodingInfo.CharsetMethods.fgCharsetCanEncodeMethod.invoke(charset, (Object[])null)) {
                    this.fCharsetEncoder = EncodingInfo.CharsetMethods.fgCharsetNewEncoderMethod.invoke(charset, (Object[])null);
                }
                else {
                    this.fHaveTriedCharsetEncoder = true;
                }
            }
            catch (Exception e) {
                this.fHaveTriedCharsetEncoder = true;
            }
        }
        if (this.fCharsetEncoder != null) {
            try {
                this.fArgsForMethod[0] = new Character(ch);
                return (boolean)EncodingInfo.CharsetMethods.fgCharsetEncoderCanEncodeMethod.invoke(this.fCharsetEncoder, this.fArgsForMethod);
            }
            catch (Exception e) {
                this.fCharsetEncoder = null;
                this.fHaveTriedCharsetEncoder = false;
            }
        }
        if (this.fCharToByteConverter == null) {
            if (this.fHaveTriedCToB || !EncodingInfo.CharToByteConverterMethods.fgConvertersAvailable) {
                return false;
            }
            if (this.fArgsForMethod == null) {
                this.fArgsForMethod = new Object[1];
            }
            try {
                this.fArgsForMethod[0] = this.javaName;
                this.fCharToByteConverter = EncodingInfo.CharToByteConverterMethods.fgGetConverterMethod.invoke(null, this.fArgsForMethod);
            }
            catch (Exception e) {
                this.fHaveTriedCToB = true;
                return false;
            }
        }
        try {
            this.fArgsForMethod[0] = new Character(ch);
            return (boolean)EncodingInfo.CharToByteConverterMethods.fgCanConvertMethod.invoke(this.fCharToByteConverter, this.fArgsForMethod);
        }
        catch (Exception e) {
            this.fCharToByteConverter = null;
            return this.fHaveTriedCToB = false;
        }
    }
    
    public static void testJavaEncodingName(String name) throws UnsupportedEncodingException {
        byte[] bTest = { 118, 97, 108, 105, 100 };
        String s = new String(bTest, name);
    }
    
    static class CharsetMethods
    {
        private static Method fgCharsetForNameMethod;
        private static Method fgCharsetCanEncodeMethod;
        private static Method fgCharsetNewEncoderMethod;
        private static Method fgCharsetEncoderCanEncodeMethod;
        private static boolean fgNIOCharsetAvailable;
        
        private CharsetMethods() {
        }
        
        static {
            EncodingInfo.CharsetMethods.fgCharsetForNameMethod = null;
            EncodingInfo.CharsetMethods.fgCharsetCanEncodeMethod = null;
            EncodingInfo.CharsetMethods.fgCharsetNewEncoderMethod = null;
            EncodingInfo.CharsetMethods.fgCharsetEncoderCanEncodeMethod = null;
            EncodingInfo.CharsetMethods.fgNIOCharsetAvailable = false;
            try {
                Class<Object> charsetClass = Class.forName("java.nio.charset.Charset");
                Class<Object> charsetEncoderClass = Class.forName("java.nio.charset.CharsetEncoder");
                EncodingInfo.CharsetMethods.fgCharsetForNameMethod = charsetClass.getMethod("forName", new Class[] { String.class });
                EncodingInfo.CharsetMethods.fgCharsetCanEncodeMethod = charsetClass.getMethod("canEncode", new Class[0]);
                EncodingInfo.CharsetMethods.fgCharsetNewEncoderMethod = charsetClass.getMethod("newEncoder", new Class[0]);
                EncodingInfo.CharsetMethods.fgCharsetEncoderCanEncodeMethod = charsetEncoderClass.getMethod("canEncode", new Class[] { Character.TYPE });
                EncodingInfo.CharsetMethods.fgNIOCharsetAvailable = true;
            }
            catch (Exception exc) {
                EncodingInfo.CharsetMethods.fgCharsetForNameMethod = null;
                EncodingInfo.CharsetMethods.fgCharsetCanEncodeMethod = null;
                EncodingInfo.CharsetMethods.fgCharsetEncoderCanEncodeMethod = null;
                EncodingInfo.CharsetMethods.fgCharsetNewEncoderMethod = null;
                EncodingInfo.CharsetMethods.fgNIOCharsetAvailable = false;
            }
        }
    }
    
    static class CharToByteConverterMethods
    {
        private static Method fgGetConverterMethod;
        private static Method fgCanConvertMethod;
        private static boolean fgConvertersAvailable;
        
        private CharToByteConverterMethods() {
        }
        
        static {
            EncodingInfo.CharToByteConverterMethods.fgGetConverterMethod = null;
            EncodingInfo.CharToByteConverterMethods.fgCanConvertMethod = null;
            EncodingInfo.CharToByteConverterMethods.fgConvertersAvailable = false;
            try {
                Class<Object> clazz = Class.forName("sun.io.CharToByteConverter");
                EncodingInfo.CharToByteConverterMethods.fgGetConverterMethod = clazz.getMethod("getConverter", new Class[] { String.class });
                EncodingInfo.CharToByteConverterMethods.fgCanConvertMethod = clazz.getMethod("canConvert", new Class[] { Character.TYPE });
                EncodingInfo.CharToByteConverterMethods.fgConvertersAvailable = true;
            }
            catch (Exception exc) {
                EncodingInfo.CharToByteConverterMethods.fgGetConverterMethod = null;
                EncodingInfo.CharToByteConverterMethods.fgCanConvertMethod = null;
                EncodingInfo.CharToByteConverterMethods.fgConvertersAvailable = false;
            }
        }
    }
}
